/*
 * Copyright (c) 1996 by Tadayoshi Maekawa, Nihon Sun Microsystems K.K.
 *
 * Permission to use, copy, and distribute this software for
 * NON-COMMERCIAL purposes and without fee is hereby granted
 * provided that this copyright notice appears in all copies.
 *
 */

package  COM.Sun.Japan.yjs.mwt;

import  java.awt.*;
import  java.awt.event.MouseEvent;

/**
 * ItEXN[pC[W CanvasB
 *
 * @version     1.4 97/01/11
 * @author      Tadayoshi Maekawa, Nihon Sun Microsystems K.K.
 */
public class BufferedCanvas extends Canvas {
    private static final int DEFAULT_WIDTH = 1200;
    private static final int DEFAULT_HEIGHT = 900;

    /**
     * ItEXN[pC[WB
     */
    Image screenBuffer;
    
    /**
     * ItEXN[̕B
     */
    int bufferWidth;

    /**
     * ItEXN[̍B
     */
    int bufferHeight;

    /**
     * ItEXN[̏c\ItZbgB
     */
    int offsetX;

    /**
     * ItEXN[̉\ItZbgB
     */
    int offsetY;

    /**
     * ftHgERXgN^B
     */
    public BufferedCanvas() {
        this(-1, -1);
    }

    /**
     * ItEXN[̕ƍɂƂRXgN^B
     * @param width ItEXN[̕
     * @param height ItEXN[̍
     */
    public BufferedCanvas(int width, int height) {
        bufferWidth = (width < 0) ? DEFAULT_WIDTH : width;
        bufferHeight = (height < 0) ? DEFAULT_HEIGHT  : height;
    }

    /**
     * ItEXN[̊mۂsB
     */
    public void addNotify() {
        super.addNotify();
        generateBuffer(bufferWidth, bufferHeight, false);
    }

    /**
     * paint \bhB
     * @param g LoX Graphics
     */
    public void paint(Graphics g) {
        Dimension d = getSize();
        g.setColor(getBackground());
        g.fillRect(0, 0, d.width, d.height);
        g.drawImage(screenBuffer, -offsetX, -offsetY, this);
    }

    /**
     * Dimension ɂƂAItEXN[TCYB
     * @param dimension VItEXN[ Dimension
     */
    public void setSizeOfBuffer(Dimension dimension) {
        if (dimension!=null)
            setSizeOfBuffer(dimension.width, dimension.height);
    }

    /**
     * ƍɂƂAItEXN[TCYB
     * @param width VItEXN[̕
     * @param height VItEXN[̍
     */
    public synchronized void setSizeOfBuffer(int width, int height) {
        generateBuffer(width, height, true);
    }

    /**
     * ItEXN[̃TCYԂB
     */
    public Dimension getSizeOfBuffer() {
        return new Dimension(bufferWidth, bufferHeight);
    }

    /**
     * ItEXN[\ɗpItZbgZbgB
     * @param x ̃ItZbg
     * @param y c̃ItZbg
     */
    public synchronized void setOffset(int x, int y) {
        offsetX = x;
        offsetY = y;
        repaint();
    }

    /**
     * ݂̃ItZbglԂB
     */
    public Point getOffset() {
        return new Point(offsetX, offsetY);
    }

    /**
     * ItEXN[ Graphics ԂB
     */
    public Graphics getBufferGraphics() {
        Graphics g = null;
        if (screenBuffer != null)
            g = screenBuffer.getGraphics();
        return g;
    }

    /**
     * ItEXN[NAB
     */
    public synchronized void clearBuffer() {
        Graphics g = getBufferGraphics();
        if (g!=null) {
            g.setColor(getBackground());
            g.fillRect(0, 0, bufferWidth, bufferHeight);
        }
        repaint();
    }

    /**
     * mouse Cxg̏B
     * @param ev mouse Cxg
     */
    protected boolean processMouseEvent(MouseEvent ev) {
        ev.translatePoint(offsetX, offsetY);
        return super.processMouseEvent(ev);
    }

    /**
     * VItEXN[̊mہB
     * @param w VItEXN[̕
     * @param h VItEXN[̍
     * @param f Rs[̃tOBtrue Ȃ΁AÂItEXN[
     * VItEXN[ɃRs[B
     */
    protected void generateBuffer(int w, int h, boolean f) {
        Image  newBuffer;
        Graphics g;
        bufferWidth = w;
        bufferHeight = h;
        newBuffer = createImage(w, h);
        if (f &&
            newBuffer != null &&
            (g = newBuffer.getGraphics()) != null &&
            screenBuffer != null) {
            g.drawImage(screenBuffer, 0, 0, this);
        }
        screenBuffer = null; // verbose, maybe.
        screenBuffer = newBuffer;
    }
}
